@@ -19,9 +19,7 @@ import lir as multilir
 
 from . import metrics
 from .lr import *
-from .bayeserror import *
 from .calibration import *
-from .ece import *
 from .generators import *
 from .util import Xn_to_Xy, Xy_to_Xn
-from .plotting import *
\ No newline at end of file
+from . import plotting
